from .workflow import (
    WorkflowComponent,
    WorkflowOverrideDefaultsComponent,
    WorkflowPermissionsComponent,
)

WORKFLOW_COMPONENTS = [
    WorkflowComponent,
    WorkflowOverrideDefaultsComponent,
    WorkflowPermissionsComponent,
]
